/* $Id: heap.c,v 1.1.1.1 2001/07/01 15:49:41 dw Exp $ */
/* Author Damian Weber, www-crypto.htw-saarland.de, License: BSD */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* heap sort for the a[0...n-1] */

/* heap elements are stored in a[1],...,a[size] */
struct heap
{
	int *a;
	int size;
};

void heap_init(struct heap *h, int max_size)
{
	h->size=0;
	h->a=(int *) malloc((max_size+1)*sizeof(int));
}

void heap_destroy(struct heap *h)
{
	free(h->a);
}


/* insert e into the heap */
void heap_insert(struct heap *h,int e)
{
	register int i,i2;
	int t;

	(h->size)++;
	i=h->size;
	h->a[h->size]=e;

	while (i>1)
	{
		i2=i>>1;
		if (h->a[i]>h->a[i2])
		{
			t=h->a[i]; h->a[i]=h->a[i2]; h->a[i2]=t;
			i=i2;
		}
		else break;
	}
}

/* find maximum in the heap */
int heap_find_max(struct heap *h)
{
	if (h->size>0) return h->a[1];
	printf("error: max on empty heap requested\n");
	return 0;
}

/* delete max from the heap */
void heap_delete_max(struct heap *h)
{
	register int i,l,r;
	int t;

	if (h->size<=0) 
	{
		printf("error: delete_max on empty heap requested\n");
		return;
	}


	h->a[1]=h->a[h->size];
	(h->size)--;

	i=1;
	while ((l=2*i)<=h->size)
	{
		r=l+1;
		if (r>h->size || h->a[l]>h->a[r])
		{
			if (h->a[i]<h->a[l])
			{
				t=h->a[i]; h->a[i]=h->a[l]; h->a[l]=t;
				i=l;
			}
			else break; /* !!! exit from loop */
		}
		else
		{
			if (h->a[i]<h->a[r])
			{
				t=h->a[i]; h->a[i]=h->a[r]; h->a[r]=t;
				i=r;
			}
			else break; /* !!! exit from loop */
		}
	}
}

/* sort array a[0],...,a[n-1] by heap sort */
void heap_sort(int n,int *a)
{
	int i;
	struct heap h;

	heap_init(&h,n);

	for (i=0;i<n;i++)
		heap_insert(&h,a[i]);

	for (i=0;i<n;i++)
	{
		a[n-1-i]=heap_find_max(&h);
		heap_delete_max(&h);
	}
	heap_destroy(&h);
}
