/* $Id: quick.c,v 1.3 2004/02/15 22:27:14 dw Exp $ */
/* Author Damian Weber, www-crypto.htw-saarland.de, License: BSD */


#include <stdio.h>

#include "sorting.h"

#undef DEBUG

/* Sedgewick's description in "Analysis of Algorithms" */
void quick_sort_r(int l, int r, int *a) 
{
	register int v,t,i,j;

	if (l<=r)
	{
		v=a[r];
		i=l-1;
		j=r;
		do
		{
			do { i++ ; } while (a[i]<v);
			do { j--; } while (a[j]>v);
			t=a[i];a[i]=a[j];a[j]=t;
		}
		while (j>i);
		a[j]=a[i]; a[i]=a[r];a[r]=t;
		quick_sort_r(l,i-1,a);
		quick_sort_r(i+1,r,a);
	}
}


/* rearrange the array such that a[l+1],...,a[k] are <=a[l]
 * and a[k+1],...,a[r] are >a[l], then swap a[k] and a[l]
 * and call quicksort(l,k) and quicksort(k+1,r)
 */

void quick_sort_r_dw(int l, int r, int *a) 
{
	register int y;
	register int j;
	register int i;
	register int t;
#ifdef DEBUG
	static int recursion=0;

	recursion++;
	if (recursion%100==0) printf("quick-recursion %4d\n",recursion);
	/*   printf("quick(%d,%d)\n",l,r); */
#endif

	if (l>=r) return; /* nothing to do */
	y=a[l];

	if (r-l<=50)
	{
		selection_sort_p(l,r,a);
		return;
	}

	i=l; j=r;
	while (i<j)
	{
		while (i<j && a[j]>=y)  j--;
		while (i<j && a[i]<=y) i++;
		t=a[i]; a[i]=a[j]; a[j]=t;
	}

	if (j!=l && i!=r)
	{
		a[l]=a[i];
		a[i]=y;
	}
	if (j==l) l++; /* a[j]>y for all j => a[l] is minimum */
	if (i==r)      /* a[i]<=y for all i => a[l] is maximum */
	{
		a[l]=a[r];
		a[r]=y;
		r--;
	}

	if (i<(l+r)/2)
	{
		if (l<i)   quick_sort_r(l,i,a);
		if (i+1<r) quick_sort_r(i+1,r,a);
	}
	else
	{
		if (i+1<r) quick_sort_r(i+1,r,a);
		if (l<i)   quick_sort_r(l,i,a);
	}
#ifdef DEBUG
	recursion--;
	if (recursion%100==0) printf("quick-recursion %4d\n",recursion);
#endif
}

/* quick sort for the a[0...n-1] */

void quick_sort(int n, int *a) 
{
	quick_sort_r(0,n-1,a);
}
